from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="cancel_token",
    version="0.1.5",
    author="Maciej Nowak",
    description="Simple CancellationToken",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Novakov/py-cancel-token",
    packages=['cancel_token'],
    package_data={
        'cancel_token': ['py.typed']
    },
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Utilities"
    ],
    include_package_data=True
)


from threading import Lock
from typing import List, Callable


class CancellationToken:
    def __init__(self):
        # type: () -> None
        self._callbacks = []      # type: List[Callable[[], None]]
        self._canceled = False
        self._completed = False
        self._lock = Lock()

    def on_cancel(self, callback):
        # type: (Callable[[], None]) -> None
        if self._canceled:
            callback()
        else:
            self._callbacks.append(callback)

    def remove_callback(self, callback):
        # type: (Callable[[], None]) -> None
        self._callbacks.remove(callback)

    def cancel(self):
        # type: () -> None
        with self._lock:
            if self._canceled:
                return

            self._canceled = True
            self._completed = True

        for f in [x for x in self._callbacks]:
            f()

    def complete(self):
        # type: () -> None
        with self._lock:
            self._completed = True

    @property
    def cancelled(self):
        # type: () -> bool
        return self._canceled

    @property
    def completed(self):
        # type: () -> bool
        return self._completed
